package cn.ecpark.service.usergw.config;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.filter.NettyWriteResponseFilter;
import org.springframework.cloud.gateway.handler.RoutePredicateHandlerMapping;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionLocator;
import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Controller;
import org.springframework.util.ReflectionUtils;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.reactive.config.WebFluxConfigurationSupport;
import org.springframework.web.reactive.result.method.AbstractHandlerMethodMapping;
import org.springframework.web.reactive.result.method.RequestMappingInfo;
import org.springframework.web.reactive.result.method.annotation.RequestMappingHandlerMapping;
import org.yaml.snakeyaml.Yaml;

import cn.ecpark.service.usergw.impl.http.Http2Dubbo;
import cn.ecpark.service.usergw.utils.Convert;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;
import reactor.netty.http.server.HttpServerRequest;
import reactor.netty.http.server.HttpServerResponse;

/**
 * ConfigGateway
 */
@Component
@Controller
@Slf4j
public class ConfigGateway implements RouteDefinitionLocator {

  // List<MediaType> mediaTypes = new ArrayList<>();

  @Autowired
  private ApplicationContext applicationContext;

  @Autowired
  private Http2Dubbo http2Dubbo;

  @Override
  public Flux<RouteDefinition> getRouteDefinitions() {
    // WebFluxConfigurationSupport a;

    Object inputStream = null;
    String[] gatewayConfigPathList = { "gateway.yml", "gateway.yaml", "Gateway.yml", "Gateway.yaml" };
    for (String gatewayConfigPath : gatewayConfigPathList) {
      inputStream = this.getClass().getClassLoader().getResourceAsStream(gatewayConfigPath);
      if (inputStream != null) {
        break;
      }
    }

    if (inputStream != null) {
      Map<String, Object> configYaml = new Yaml().load((InputStream) inputStream);
      if (configYaml != null) {

        // streaming-media-types 这个先不管
        // NettyWriteResponseFilter nwrf =
        // applicationContext.getBean(NettyWriteResponseFilter.class);
        // try {
        // Field field = nwrf.getClass().getDeclaredField("streamingMediaTypes");
        // field.setAccessible(true);
        // List<MediaType> mts = new ArrayList<MediaType>();
        // mts.add(MediaType.TEXT_EVENT_STREAM);
        // mts.add(MediaType.APPLICATION_STREAM_JSON);
        // field.set(nwrf, mts);
        // } catch (Exception e) {
        // log.error(e.toString());
        // }

        Map<String, Object> configDefault = (Map<String, Object>) configYaml.get("default");
        Map<String, Object> configDubbo = (Map<String, Object>) configYaml.get("dubbo");

        this.createHttp2Dubbo(configDubbo);
        return Flux.fromIterable(this.getConfigDefault(configDefault));
      }
    }
    return null;
    // return Flux.fromIterable(it);
  }

  private List<RouteDefinition> getConfigDefault(Map<String, Object> configDefault) {
    if (configDefault != null) {
      List<RouteDefinition> RDList = new ArrayList<>();

      List<FilterDefinition> filters = new ArrayList<>();
      List<PredicateDefinition> predicates = new ArrayList<>();

      // default-filters: 下的相关设置
      Object unknownDefaultFilters = configDefault.get("default-filters");
      if (unknownDefaultFilters != null) {
        List<String> defaultFiltersYaml = (ArrayList<String>) unknownDefaultFilters;
        for (String filterString : defaultFiltersYaml) {
          filters.add(new FilterDefinition(filterString));
        }
      }

      Object unknownRoutes = configDefault.get("routes");
      if (unknownRoutes != null) {
        List<LinkedHashMap> routes = (ArrayList<LinkedHashMap>) unknownRoutes;
        for (LinkedHashMap iter : routes) {
          RouteDefinition rd = new RouteDefinition();
          // 设置id
          Object id = iter.get("id");
          if (id != null) {
            rd.setId((String) id);
          }

          // 设置uri
          Object uri = iter.get("uri");
          if (uri != null) {
            rd.setUri(URI.create((String) uri));
          }

          // 设置uri
          Object order = iter.get("order");
          if (order != null) {
            rd.setOrder((int) order);
          }

          // predicates: 下的相关属性
          List<String> unknownPredicatesYaml = (ArrayList<String>) iter.get("predicates");
          if (unknownPredicatesYaml != null) {
            List<String> predicatesYaml = (ArrayList<String>) unknownPredicatesYaml;
            for (String predicateString : predicatesYaml) {
              predicates.add(new PredicateDefinition(predicateString));
            }
          }

          // filters: 下的相关属性
          List<String> unknownFiltersYaml = (ArrayList<String>) iter.get("filters");
          if (unknownFiltersYaml != null) {
            List<String> filtersYaml = (ArrayList<String>) unknownFiltersYaml;
            for (String filterString : filtersYaml) {
              filters.add(new FilterDefinition(filterString));
            }
          }

          rd.setPredicates(predicates);
          rd.setFilters(filters);

          RDList.add(rd);
        }

        return RDList;
      }
    }

    return null;
  }

  private void createHttp2Dubbo(Map<String, Object> configDubbo) {

    try {
  
      // RequestMappingHandlerMapping requestMapping = (RequestMappingHandlerMapping) applicationContext
      //     .getBean("requestMappingHandlerMapping");
          
      // // Method targetMethod = ReflectionUtils.findMethod(Http2Dubbo.class, "H2DTest",
      // // HttpServerRequest.class, HttpServerResponse.class); // 找到处理该路由的方法

      // requestMapping.registerMapping(RequestMappingInfo.paths("/test/xixi").methods(RequestMethod.POST).build(),
      //     http2Dubbo, Http2Dubbo.class.getDeclaredMethod("H2DTest"));

      // 引用远程服务
      // try {
      // ReferenceConfig<GenericService> reference = new
      // ReferenceConfig<GenericService>(); // 该实例很重量，里面封装了所有与注册中心及服务提供方连接，请缓存

      // reference.setInterface("com.xxx.XxxService"); // 弱类型接口名
      // reference.setVersion("2.0.0");
      // reference.setGeneric(true); // 声明为泛化接口
      // GenericService gs = reference.get();

    } catch (Exception e) {
      // TODO: 任何错误都直接终止退出
      SpringApplication.exit(applicationContext);
    }

    // gs.$invoke(method, parameterTypes, args)

  }

  private static void callMethod(ReferenceConfig ref, String name, String params) throws Exception {
    Method method = ref.getClass().getMethod("set" + Convert.firstUpperCase(name));
    method.invoke(ref, method.getParameterTypes(), params);
  }
}